/*
 * XdccDownloader - MessageCleanerTest.java - Copyright © 2013 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.irc.util;

import static java.util.EnumSet.of;
import static net.pterodactylus.irc.util.MessageCleaner.Attributes.bold;
import static net.pterodactylus.irc.util.MessageCleaner.Attributes.clear;
import static net.pterodactylus.irc.util.MessageCleaner.Attributes.color;
import static net.pterodactylus.irc.util.MessageCleaner.Attributes.italics;
import static net.pterodactylus.irc.util.MessageCleaner.Attributes.reverse;
import static net.pterodactylus.irc.util.MessageCleaner.Attributes.underline;
import static net.pterodactylus.irc.util.MessageCleaner.getDefaultInstance;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;

import java.util.Collections;

import net.pterodactylus.irc.util.MessageCleaner.Attributes;

import org.junit.Test;

/**
 * Tests for {@link MessageCleaner}.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class MessageCleanerTest {

	private final MessageCleaner messageCleaner = getDefaultInstance();

	@Test
	public void removeBoldFormatting() {
		assertThat(messageCleaner.clean("Text\u0002bold", of(bold)), is("Textbold"));
	}

	@Test
	public void boldFormattingIsNotRemovedUnwanted() {
		assertThat(messageCleaner.clean("Text\u0002bold", Collections.<Attributes>emptySet()), is("Text\u0002bold"));
	}

	@Test
	public void removeColorFormatting() {
		assertThat(messageCleaner.clean("Text\u0003!color", of(color)), is("Text!color"));
		assertThat(messageCleaner.clean("Text\u0003color", of(color)), is("Textcolor"));
		assertThat(messageCleaner.clean("Text\u00034color", of(color)), is("Textcolor"));
		assertThat(messageCleaner.clean("Text\u00034!color", of(color)), is("Text!color"));
		assertThat(messageCleaner.clean("Text\u000314color", of(color)), is("Textcolor"));
		assertThat(messageCleaner.clean("Text\u000318color", of(color)), is("Text8color"));
		assertThat(messageCleaner.clean("Text\u000399color", of(color)), is("Textcolor"));
		assertThat(messageCleaner.clean("Text\u00031,!color", of(color)), is("Text!color"));
		assertThat(messageCleaner.clean("Text\u00031,color", of(color)), is("Textcolor"));
		assertThat(messageCleaner.clean("Text\u00031,1!color", of(color)), is("Text!color"));
		assertThat(messageCleaner.clean("Text\u00031,1color", of(color)), is("Textcolor"));
		assertThat(messageCleaner.clean("Text\u00031,14color", of(color)), is("Textcolor"));
		assertThat(messageCleaner.clean("Text\u00031,18color", of(color)), is("Text8color"));
		assertThat(messageCleaner.clean("Text\u00031,99color", of(color)), is("Textcolor"));
		assertThat(messageCleaner.clean("Text\u000314,color", of(color)), is("Textcolor"));
		assertThat(messageCleaner.clean("Text\u000314,1color", of(color)), is("Textcolor"));
		assertThat(messageCleaner.clean("Text\u000314,14color", of(color)), is("Textcolor"));
		assertThat(messageCleaner.clean("Text\u000314,18color", of(color)), is("Text8color"));
		assertThat(messageCleaner.clean("Text\u000314,99color", of(color)), is("Textcolor"));
		assertThat(messageCleaner.clean("Text\u000399,99color", of(color)), is("Textcolor"));
		assertThat(messageCleaner.clean("Text\u000399,9color", of(color)), is("Textcolor"));
		assertThat(messageCleaner.clean("Text\u00039color", of(color)), is("Textcolor"));
		assertThat(messageCleaner.clean("Text\u00039,color", of(color)), is("Textcolor"));
	}

	@Test
	public void colorFormattingIsNotRemovedUnwanted() {
		assertThat(messageCleaner.clean("Text\u00034color", Collections.<Attributes>emptySet()), is("Text\u00034color"));
		assertThat(messageCleaner.clean("Text\u000314color", Collections.<Attributes>emptySet()), is("Text\u000314color"));
		assertThat(messageCleaner.clean("Text\u000318color", Collections.<Attributes>emptySet()), is("Text\u000318color"));
		assertThat(messageCleaner.clean("Text\u000399color", Collections.<Attributes>emptySet()), is("Text\u000399color"));
		assertThat(messageCleaner.clean("Text\u00031,color", Collections.<Attributes>emptySet()), is("Text\u00031,color"));
		assertThat(messageCleaner.clean("Text\u00031,1color", Collections.<Attributes>emptySet()), is("Text\u00031,1color"));
		assertThat(messageCleaner.clean("Text\u00031,14color", Collections.<Attributes>emptySet()), is("Text\u00031,14color"));
		assertThat(messageCleaner.clean("Text\u00031,18color", Collections.<Attributes>emptySet()), is("Text\u00031,18color"));
		assertThat(messageCleaner.clean("Text\u00031,99color", Collections.<Attributes>emptySet()), is("Text\u00031,99color"));
		assertThat(messageCleaner.clean("Text\u000314,color", Collections.<Attributes>emptySet()), is("Text\u000314,color"));
		assertThat(messageCleaner.clean("Text\u000314,1color", Collections.<Attributes>emptySet()), is("Text\u000314,1color"));
		assertThat(messageCleaner.clean("Text\u000314,14color", Collections.<Attributes>emptySet()), is("Text\u000314,14color"));
		assertThat(messageCleaner.clean("Text\u000314,18color", Collections.<Attributes>emptySet()), is("Text\u000314,18color"));
		assertThat(messageCleaner.clean("Text\u000314,99color", Collections.<Attributes>emptySet()), is("Text\u000314,99color"));
		assertThat(messageCleaner.clean("Text\u000399,99color", Collections.<Attributes>emptySet()), is("Text\u000399,99color"));
		assertThat(messageCleaner.clean("Text\u00039color", Collections.<Attributes>emptySet()), is("Text\u00039color"));
	}

	@Test
	public void removeClearFormatting() {
		assertThat(messageCleaner.clean("Text\u000fclear", of(clear)), is("Textclear"));
	}

	@Test
	public void clearFormattingIsNotRemovedUnwanted() {
		assertThat(messageCleaner.clean("Text\u000fclear", Collections.<Attributes>emptySet()), is("Text\u000fclear"));
	}

	@Test
	public void removeUnderlineFormatting() {
		assertThat(messageCleaner.clean("Text\u001funderline", of(underline)), is("Textunderline"));
	}

	@Test
	public void underlineFormattingIsNotRemovedUnwanted() {
		assertThat(messageCleaner.clean("Text\u001funderline", Collections.<Attributes>emptySet()), is("Text\u001funderline"));
	}

	@Test
	public void removeReverseFormatting() {
		assertThat(messageCleaner.clean("Text\u0016reverse", of(reverse)), is("Textreverse"));
	}

	@Test
	public void reverseFormattingIsNotRemovedUnwanted() {
		assertThat(messageCleaner.clean("Text\u0016reverse", Collections.<Attributes>emptySet()), is("Text\u0016reverse"));
	}

	@Test
	public void removeItalicFormatting() {
		assertThat(messageCleaner.clean("Text\u001ditalics", of(italics)), is("Textitalics"));
	}

	@Test
	public void italicFormattingIsNotRemovedUnwanted() {
		assertThat(messageCleaner.clean("Text\u001ditalics", Collections.<Attributes>emptySet()), is("Text\u001ditalics"));
	}

	@Test
	public void removeEverything() {
		assertThat(messageCleaner.clean("a\u0002b\u00031,49\u001dd\u001fe\u0016f\u000fg"), is("ab9defg"));
	}

}
